package discord

import (
	"context"
	"encoding/json"
	"fmt"
	"strings"
	"time"

	"gitea.dwysokinski.me/twhelp/dcbot/internal/discord/internal/discordi18n"
	"gitea.dwysokinski.me/twhelp/dcbot/internal/domain"
	"github.com/bwmarrin/discordgo"
	"github.com/nicksnyder/go-i18n/v2/i18n"
	"go.uber.org/zap"
)

type coordsCommand struct {
	localizer *discordi18n.Localizer
	logger    *zap.Logger
	coordsSvc CoordsService
	choiceSvc *choiceService
}

func (c *coordsCommand) name() string {
	return "coords"
}

func (c *coordsCommand) register(s *discordgo.Session) error {
	if err := c.create(s); err != nil {
		return err
	}

	s.AddHandler(c.handle)

	return nil
}

func (c *coordsCommand) create(s *discordgo.Session) error {
	dm := true

	optionVersionDefault, optionVersionLocalizations, err := c.localizer.LocalizeDiscord("cmd.coords.option.version.description")
	if err != nil {
		return err
	}

	optionServerDefault, optionServerLocalizations, err := c.localizer.LocalizeDiscord("cmd.coords.option.server.description")
	if err != nil {
		return err
	}

	optionCoordsDefault, optionCoordsLocalizations, err := c.localizer.LocalizeDiscord("cmd.coords.option.coords.description")
	if err != nil {
		return err
	}

	cmdDescriptionDefault, cmdDescriptionLocalizations, err := c.localizer.LocalizeDiscord("cmd.coords.description")
	if err != nil {
		return err
	}

	versionChoices, err := c.choiceSvc.versions(context.Background())
	if err != nil {
		return err
	}

	_, err = s.ApplicationCommandCreate(s.State.User.ID, "", &discordgo.ApplicationCommand{
		Name:                     c.name(),
		Description:              cmdDescriptionDefault,
		DescriptionLocalizations: &cmdDescriptionLocalizations,
		DMPermission:             &dm,
		Options: []*discordgo.ApplicationCommandOption{
			{
				Name:                     "version",
				Description:              optionVersionDefault,
				DescriptionLocalizations: optionVersionLocalizations,
				Type:                     discordgo.ApplicationCommandOptionString,
				Choices:                  versionChoices,
				Required:                 true,
			},
			{
				Name:                     "server",
				Description:              optionServerDefault,
				DescriptionLocalizations: optionServerLocalizations,
				Type:                     discordgo.ApplicationCommandOptionString,
				Required:                 true,
			},
			{
				Name:                     "coords",
				Description:              optionCoordsDefault,
				DescriptionLocalizations: optionCoordsLocalizations,
				Type:                     discordgo.ApplicationCommandOptionString,
				Required:                 true,
				MaxLength:                domain.CoordsStringMaxLength,
			},
		},
	})
	if err != nil {
		return err
	}

	return nil
}

func (c *coordsCommand) handle(s *discordgo.Session, i *discordgo.InteractionCreate) {
	ctx := context.Background()

	//nolint:exhaustive
	switch i.Type {
	case discordgo.InteractionApplicationCommand:
		c.handleCommand(ctx, s, i)
	case discordgo.InteractionMessageComponent:
		c.handleMessageComponent(ctx, s, i)
	}
}

const coordsTranslationVillagesPerPage = 7

func (c *coordsCommand) handleCommand(ctx context.Context, s *discordgo.Session, i *discordgo.InteractionCreate) {
	cmdData := i.ApplicationCommandData()

	if cmdData.Name != c.name() {
		return
	}

	locale := string(i.Locale)

	params, err := c.optionsToTranslateCoordsParams(cmdData.Options)
	if err != nil {
		_ = s.InteractionRespond(i.Interaction, &discordgo.InteractionResponse{
			Type: discordgo.InteractionResponseChannelMessageWithSource,
			Data: &discordgo.InteractionResponseData{
				Content: c.localizer.LocalizeError(locale, err),
			},
		})
		return
	}

	res, err := c.coordsSvc.Translate(ctx, params)
	if err != nil {
		_ = s.InteractionRespond(i.Interaction, &discordgo.InteractionResponse{
			Type: discordgo.InteractionResponseChannelMessageWithSource,
			Data: &discordgo.InteractionResponseData{
				Content: c.localizer.LocalizeError(locale, err),
			},
		})
		return
	}

	var builderDescription strings.Builder
	for i, v := range res.Villages {
		if i > 0 {
			builderDescription.WriteString("\n")
		}
		builderDescription.WriteString(v.FullName)
	}

	titleMessageID := "cmd.coords.embed.title"
	title, err := c.localizer.Localize(locale, &i18n.LocalizeConfig{MessageID: titleMessageID})
	if err != nil {
		c.logger.Error("no message with the specified id", zap.String("id", titleMessageID), zap.Error(err))
		_ = s.InteractionRespond(i.Interaction, &discordgo.InteractionResponse{
			Type: discordgo.InteractionResponseChannelMessageWithSource,
			Data: &discordgo.InteractionResponseData{
				Content: c.localizer.LocalizeError(locale, err),
			},
		})
		return
	}

	_ = s.InteractionRespond(i.Interaction, &discordgo.InteractionResponse{
		Type: discordgo.InteractionResponseChannelMessageWithSource,
		Data: &discordgo.InteractionResponseData{
			Embeds: []*discordgo.MessageEmbed{
				{
					Type:        discordgo.EmbedTypeRich,
					Title:       title,
					Description: builderDescription.String(),
					Timestamp:   formatTimestamp(time.Now()),
					Footer: &discordgo.MessageEmbedFooter{
						Text: fmt.Sprintf("Page %d/%d", res.Page, res.MaxPage),
					},
				},
			},
			Components: []discordgo.MessageComponent{
				discordgo.ActionsRow{
					Components: []discordgo.MessageComponent{
						discordgo.Button{
							Style:    discordgo.PrimaryButton,
							CustomID: "coords-" + res.PrevPageID.String(),
							Label:    "Previous page",
							Disabled: res.PrevPageID.IsZero(),
						},
						discordgo.Button{
							Style:    discordgo.PrimaryButton,
							CustomID: "coords-" + res.NextPageID.String(),
							Label:    "Next page",
							Disabled: res.NextPageID.IsZero(),
						},
					},
				},
			},
		},
	})
}

func (c *coordsCommand) optionsToTranslateCoordsParams(
	options []*discordgo.ApplicationCommandInteractionDataOption,
) (domain.TranslateCoordsParams, error) {
	version := ""
	server := ""
	coordsStr := ""

	for _, opt := range options {
		switch opt.Name {
		case "version":
			version = opt.StringValue()
		case "server":
			server = opt.StringValue()
		case "coords":
			coordsStr = opt.StringValue()
		}
	}

	return domain.NewTranslateCoordsParams(version, server, coordsStr, coordsTranslationVillagesPerPage)
}

func (c *coordsCommand) handleMessageComponent(ctx context.Context, s *discordgo.Session, i *discordgo.InteractionCreate) {
	cmpData := i.MessageComponentData()

	b, _ := json.MarshalIndent(i, "", "\t")
	fmt.Println(cmpData, string(b))

	if cmpData.ComponentType != discordgo.ButtonComponent {
		return
	}
}
