package domain

import (
	"fmt"
	"math"
	"regexp"
)

type CoordsStringTooLongError struct {
	Str string
	Max int
}

var _ TranslatableError = CoordsStringTooLongError{}

func (e CoordsStringTooLongError) Error() string {
	return fmt.Sprintf("coords string is too long (%d/%d)", len(e.Str), e.Max)
}

func (e CoordsStringTooLongError) Slug() string {
	return "coords-string-too-long"
}

func (e CoordsStringTooLongError) Params() map[string]any {
	return map[string]any{
		"Str": e.Str,
		"Max": e.Max,
	}
}

type NoCoordsFoundInStringError struct {
	Str string
}

var _ TranslatableError = NoCoordsFoundInStringError{}

func (e NoCoordsFoundInStringError) Error() string {
	return fmt.Sprintf("no coords found in string: '%s'", e.Str)
}

func (e NoCoordsFoundInStringError) Slug() string {
	return "no-coords-found-in-string"
}

func (e NoCoordsFoundInStringError) Params() map[string]any {
	return map[string]any{
		"Str": e.Str,
	}
}

type TranslateCoordsParams struct {
	versionCode string
	serverKey   string
	coords      []string
	perPage     int32
	maxPage     int32
}

var coordsRegex = regexp.MustCompile(`(\d+)\|(\d+)`)

const CoordsStringMaxLength = 1000

func NewTranslateCoordsParams(versionCode, serverKey, coordsStr string, perPage int32) (TranslateCoordsParams, error) {
	if versionCode == "" {
		return TranslateCoordsParams{}, RequiredError{Field: "VersionCode"}
	}

	if serverKey == "" {
		return TranslateCoordsParams{}, RequiredError{Field: "ServerKey"}
	}

	if len(coordsStr) > CoordsStringMaxLength {
		return TranslateCoordsParams{}, CoordsStringTooLongError{
			Max: CoordsStringMaxLength,
			Str: coordsStr,
		}
	}

	coords := uniq(coordsRegex.FindAllString(coordsStr, -1))
	if len(coords) == 0 {
		return TranslateCoordsParams{}, NoCoordsFoundInStringError{
			Str: coordsStr,
		}
	}

	return TranslateCoordsParams{
		versionCode: versionCode,
		serverKey:   serverKey,
		coords:      coords,
		perPage:     perPage,
		maxPage:     int32(math.Ceil(float64(len(coords)) / float64(perPage))),
	}, nil
}

func (c TranslateCoordsParams) VersionCode() string {
	return c.versionCode
}

func (c TranslateCoordsParams) ServerKey() string {
	return c.serverKey
}

func (c TranslateCoordsParams) Coords() []string {
	return c.coords
}

func (c TranslateCoordsParams) PerPage() int32 {
	return c.perPage
}

func (c TranslateCoordsParams) MaxPage() int32 {
	return c.maxPage
}

type TranslateCoordsResult struct {
	Villages     []Village
	Untranslated []string
	Page         int32
	MaxPage      int32
	NextPageID   PaginationPageID
	PrevPageID   PaginationPageID
}
